<?php

namespace LicenseKeys\Controllers;

use Exception;
use WPMVC\Log;
use WPMVC\Request;
use WPMVC\MVC\Controller;
use LicenseKeys\Models\LicenseKey;
/**
 * AccountController controller.
 * Handles all account related business logic.
 *
 * @see https://gist.github.com/claudiosanches/a79f4e3992ae96cb821d3b357834a005
 * @author Cami Mostajo <info@10quality.com>
 * @copyright 10 Quality <http://www.10quality.com/>
 * @license GPLv3
 * @package LicenseKeys
 * @version 1.2.4
 */
class AccountController extends Controller
{
    /**
     * Account's license keys endpoint key.
     * @since 1.0.0
     *
     * @var string
     */
    const ENDPOINT = 'license-keys';
    /**
     * Account's view license key endpoint key.
     * @since 1.0.0
     *
     * @var string
     */
    const VIEW_ENDPOINT = 'view-license-key';
    /**
     * Flushes rewrite rules.
     * Plugin Activation.
     * @since 1.0.0
     */
    public function flush()
    {
        flush_rewrite_rules();
    }
    /**
     * Adds endpoint rewrite rule.
     * Action "init"
     * @since 1.0.0
     *
     * @see https://developer.wordpress.org/reference/functions/add_rewrite_endpoint/
     */
    public function add_endpoint()
    {
        add_rewrite_endpoint( self::ENDPOINT, EP_ROOT | EP_PAGES );
        add_rewrite_endpoint( self::VIEW_ENDPOINT, EP_ROOT | EP_PAGES );
    }
    /**
     * Returns query vars with endpoint considered.
     * Filter "query_vars"
     * @since 1.0.0
     *
     * @return
     */
    public function query_vars( $vars = [] )
    {
        $vars[] = self::ENDPOINT;
        $vars[] = self::VIEW_ENDPOINT;
        return $vars;
    }
    /**
     * Retuns endpoint title.
     * Filter "the_title"
     * @since 1.0.0
     *
     * @global object $wp_query Wordpress query string manager.
     *
     * @param string $title Current title.
     *
     * @return string
     */
    public function title( $title )
    {
        global $wp_query;
        if ( ! is_admin()
            && is_main_query()
            && in_the_loop()
            && is_account_page()
        ) {
            if ( isset( $wp_query->query_vars[ self::ENDPOINT ] ) ) {
                $title = __( 'License Keys', 'woo-license-keys' );
                remove_filter( 'the_title', [ $this, 'title' ] );
            } else if ( isset( $wp_query->query_vars[ self::VIEW_ENDPOINT ] ) ) {
                $title = __( 'License Key', 'woo-license-keys' );
                remove_filter( 'the_title', [ $this, 'title' ] );
            }
        }
        return $title;
    }
    /**
     * Retuns account menu items with "License Keys" option added.
     * Filter "woocommerce_account_menu_items"
     * @since 1.0.0
     *
     * @param array $menu Menu items.
     *
     * @return array
     */
    public function menu_items( $menu = [] )
    {
        $items = [];
        foreach ( $menu as $key => $value ) {
            $items[$key] = $value;
            // Add license keys menu option
            if ( $key === 'orders' ) {
                $items[self::ENDPOINT] = __( 'License Keys', 'woo-license-keys' );
            }
        }
        return $items;
    }
    /**
     * Displays endpoint.
     * Action "woocommerce_account_[...]_endpoint"
     * @since 1.0.0
     * @since 1.2.1 Added enqueue hooks.
     * @since 1.2.3 Enqueue dashicons/
     *
     * @global object $wpdb Wordpress Database accessor.
     *
     * @see https://developer.wordpress.org/reference/functions/get_current_user_id/
     * @see https://docs.woocommerce.com/wc-apidocs/function-wc_get_order_item_meta.html
     */
    public function endpoint()
    {
        global $wpdb;
        $license_keys = [];
        // Prepare data
        try {
            $license_keys = LicenseKey::fromUser( get_current_user_id() );
        } catch ( Exception $e ) {
            Log::error( $e );
        }
        wp_enqueue_style( 'dashicons' );
        do_action( 'woocommerce_license_key_enqueue' );
        do_action( 'woocommerce_license_keys_page_enqueue' );
        // Display template
        wc_get_template(
            'myaccount/license-keys.php',
            ['license_keys' => $license_keys],
            null,
            __DIR__.'/../../templates/'
        );
    }
    /**
     * Displays view endpoint.
     * Action "woocommerce_account_[...]_endpoint"
     * @since 1.0.0
     * @since 1.0.4 Added customization hooks.
     * @since 1.1.0 Added hook actions and filters for extended customization.
     * @since 1.1.2 Added is_desktop flag.
     * @since 1.2.1 Deactivation non-object fix.
     * @since 1.2.1 Added enqueue hooks.
     * @since 1.2.4 Bug fixes.
     *
     * @global object $wpdb Wordpress Database accessor.
     */
    public function view_endpoint()
    {
        global $wpdb;
        $license_key = null;
        // Prepare data
        try {
            $license_key = isset( $_GET['key'] )
                ? LicenseKey::findByUserKey(
                    get_current_user_id(),
                    sanitize_text_field( Request::input( 'key' ) )
                )
                : LicenseKey::findByUserIndex(
                    get_current_user_id(),
                    intval( Request::input( 'item' ) ),
                    intval( Request::input( 'index', 0 ) )
                );
            $license_key = apply_filters( 'woocommerce_license_key', $license_key );
            // Deactivations?
            if ( isset( $_GET['deactivate'] )
                && $license_key
                && $license_key->deactivate( intval( Request::input( 'deactivate' ) ) )
            ) {
                do_action( 'woocommerce_license_key_activation_deactivated', $license_key, Request::input( 'deactivate' ) );
                wc_get_template(
                    'notices/success.php',
                    ['messages' => [
                        sprintf( __( 'Activation ID:%d has been deactivated.', 'woo-license-keys' ), Request::input( 'deactivate' ) )
                    ]]
                );
            }
            // Display
            if ( $license_key ) {
                // Apply hook actions
                if ( isset( $_GET['hook_action'] ) && ! empty( $_GET['hook_action'] ) )
                    do_action( 'woocommerce_license_key_hook_' . $_GET['hook_action'], $license_key );
                if ( isset( $_GET['hook_filter'] ) && ! empty( $_GET['hook_filter'] ) )
                    $license_key = apply_filters( 'woocommerce_license_key_hook_' . $_GET['hook_filter'], $license_key );
                wp_enqueue_style( 'dashicons' );
                do_action( 'woocommerce_license_key_enqueue' );
                do_action( 'woocommerce_license_key_page_enqueue' );
                // Display template
                wc_get_template(
                    'myaccount/view-license-key.php',
                    [
                        'license_key'   => $license_key,
                        'is_desktop'    => get_post_meta( $license_key->product->get_id(), '_desktop', true ) === 'yes',
                    ],
                    null,
                    __DIR__.'/../../templates/'
                );
                return;
            }
        } catch ( Exception $e ) {
            Log::error( $e );
        }
        // 404 Error
        ?><script type="text/javascript">window.location = '<?= home_url( '/license-not-found' ) ?>';</script><?php
    }
    /**
     * Enqueues assets for view_endpoint.
     * Action "woocommerce_license_key_page_enqueue"
     * @since 1.2.1
     * 
     * @global LicenseKeys\Main $licensekeys
     */
    public function view_enqueue()
    {
        global $licensekeys;
        wp_enqueue_script(
            'clipboard',
            assets_url( 'js/clipboard.min.js', __DIR__ ),
            [],
            '2.0.4',
            true
        );
        wp_enqueue_script(
            'license-keys-app',
            assets_url( 'js/app.js', __DIR__ ),
            ['clipboard', 'jquery'],
            $licensekeys->config->get( 'version' ),
            true
        );
    }
}