<?php

namespace LicenseKeys\Controllers;

use WC_Product_License_Key;
use Keygen\Keygen;
use WPMVC\Log;
use WPMVC\MVC\Controller;
use LicenseKeys\Models\LicenseKey;
/**
 * WooCommerceController controller.
 * Handles all WooCommerce related business logic and hooks.
 *
 * @author Cami Mostajo <info@10quality.com>
 * @copyright 10 Quality <http://www.10quality.com/>
 * @license GPLv3
 * @package LicenseKeys
 * @version 1.2.9
 */
class WooCommerceController extends Controller
{
    /**
     * Retuns list of product types available.
     * Filter "product_type_selector"
     * @since 1.0.0
     *
     * @link http://hookr.io/filters/product_type_selector/
     *
     * @param array $types Product types.
     *
     * @return array
     */
    public function product_type_selector( $types = [] )
    {
        $types[WC_Product_License_Key::TYPE] = __( 'License Key', 'woo-license-keys' );
        return $types;
    }
    /**
     * Returns list of product tabs.
     * Filter "woocommerce_product_data_tabs"
     * @since 1.0.0
     * @since 1.2.9 Support for multiple license key product types.
     *
     * @link http://hookr.io/filters/woocommerce_product_data_tabs/
     *
     * @param array $tabs Product tabs.
     *
     * @return array
     */
    public function product_data_tabs( $tabs = [] )
    {
        $types = apply_filters( 'woocommerce_license_key_types', [] );
        // Hide "attribute"
        $classes = apply_filters( 'woocommerce_license_key_admin_attribute_tab_class', array_map(
            function( $type ) {
                return 'hide_if_' . $type;
            },
            $types
        ) );
        foreach ( $classes as $class ) {
            $tabs['attribute']['class'][] = $class;
        }
        // Show
        $classes = apply_filters( 'woocommerce_license_key_admin_general_tab_class', array_map(
            function( $type ) {
                return 'show_if_' . $type;
            },
            $types
        ) );
        foreach ( $classes as $class ) {
            $tabs['general']['class'][] = $class;
        }
        // Add license key tab
        $tabs[WC_Product_License_Key::TYPE] = [
            'label'     => __( 'License Key', 'woo-license-keys' ),
            'target'    => 'license_key_product_data',
            'class'     => apply_filters( 'woocommerce_license_key_admin_license_key_tab_class', array_map(
                            function( $type ) {
                                return 'show_if_' . $type;
                            },
                            $types
                        ) ),
        ];
        return $tabs;
    }
    /**
     * Displays data panels.
     * Action "woocommerce_product_data_panels"
     * @since 1.0.0
     *
     * @link http://hookr.io/actions/woocommerce_product_data_panels/
     *
     * @global Bridge $licensekeys Main class.
     *
     * @param array $panels Product panels.
     */
    public function product_data_panels( $panels = [] )
    {
        global $licensekeys;
        $this->view->show( 'admin.woocommerce.product-panel', [
            'main'      => &$licensekeys,
            'types'     => apply_filters( 'woocommerce_license_key_types', [] ),
        ] );
    }
    /**
     * Saves product meta.
     * Action "woocommerce_process_product_meta_license_key"
     * @since 1.0.0
     * @since 1.1.0 Added meta to identify license keys.
     * @since 1.1.2 Added _desktop.
     * @since 1.2.6 Added _sold_individually.
     *
     * @link http://hookr.io/actions/woocommerce_process_product_meta/
     *
     * @param int $post_id Product ID.
     */
    public function save_product_meta( $post_id )
    {
        update_post_meta( $post_id, '_10q_lk', WC_Product_License_Key::TYPE );
        // Expire
        update_post_meta( $post_id, '_expire', isset( $_POST['_expire'] ) ? 'yes' : 'no' );
        if ( isset( $_POST['_expire'] ) ) {
            update_post_meta( $post_id, '_expire_interval', trim( $_POST['_expire_interval'] ) );
            update_post_meta( $post_id, '_expire_value', trim( $_POST['_expire_value'] ) );
        }
        // Desktop
        update_post_meta( $post_id, '_desktop', isset( $_POST['_desktop'] ) ? 'yes' : 'no' );
        // Override sold individually
        update_post_meta( $post_id, '_sold_individually', isset( $_POST['_sold_individually_override'] ) ? 'yes' : 'no' );
    }
    /**
     * Displays template for "license_key" product type.
     * Action "woocommerce_license_key_add_to_cart"
     * @since 1.0.0
     */
    public function add_to_cart_template()
    {
        wc_get_template( 'single-product/add-to-cart/simple.php' );
    }
    /**
     * Generates license keys.
     * Action "woocommerce_order_status_completed"
     * @since 1.0.0
     * @since 1.0.2 Log failed license key creation.
     * @since 1.0.4 Added customization hooks.
     * @since 1.1.0 Prevent for generating multiple on re-completion.
     * @since 1.2.9 Extend hooks and flag check to prevent extra processing.
     *
     * @link https://docs.woocommerce.com/wc-apidocs/class-WC_Abstract_Order.html#_get_items
     * @link https://docs.woocommerce.com/wc-apidocs/class-WC_Order_Item.html
     *
     * @param int $order_id Order ID.
     */
    public function order_completed( $order_id )
    {
        try {
            // Get order
            $order = wc_get_order( $order_id );
            // Loop items
            foreach ( $order->get_items() as $item ) {
                // Check if key must be generated or not
                $generate = wc_get_order_item_meta( $item->get_id(), '_license_key_generate', true );
                if ( $generate === null || $generate === '' ) {
                    $product = wc_get_product( $item->get_product_id() );
                    if ( wc_is_license_key( $product ) ) {
                        // Prepare expire date, limit and offline
                        $expire = null;
                        $limit = null;
                        $offline = null;
                        if ( get_post_meta( $item->get_product_id(), '_expire', true ) === 'yes' ) {
                            $value = intval( get_post_meta( $item->get_product_id(), '_expire_value', true ) );
                            $interval = get_post_meta( $item->get_product_id(), '_expire_interval', true );
                            $expire = apply_filters(
                                'woocommerce_license_key_expire_time',
                                strtotime( '+'.$value.' '.$interval ),
                                $interval,
                                $value
                            );
                        }
                        $limit = apply_filters( 'woocommerce_license_key_creation_limit', $limit, $item );
                        $offline = apply_filters( 'woocommerce_license_key_creation_offline', $offline, $item );
                        // Check if keys weren't created before
                        $license_keys = LicenseKey::fromOrderItem( $order_id, $item->get_id() );
                        // Generate license keys
                        for ( $i = count( $license_keys ); $i < $item->get_quantity(); ++$i ) {
                            wc_add_order_item_meta( $item->get_id(), '_license_key', apply_filters(
                                'woocommerce_license_key_meta_value',
                                [
                                    'code'      => Keygen::alphanum( apply_filters( 'woocommerce_license_key_length', 26 ) )->generate(),
                                    'expire'    => $expire,
                                    'uses'      => [],
                                    'limit'     => $limit,
                                    'offline'   => $offline,
                                ],
                                $item,
                                $product,
                                $order
                            ) );
                        }
                        // Turn generate flag off
                        wc_add_order_item_meta(
                            $item->get_id(),
                            '_license_key_generate',
                            apply_filters( 'woocommerce_license_keys_generate_flag', false, $item )
                        );
                        do_action( 'woocommerce_license_keys_generated', $order_id, $item );
                    }
                }
                // Is there an action to call from other plugins ?
                $action = wc_get_order_item_meta( $item->get_id(), '_license_key_action', true );
                if ( $action !== null )
                    do_action( 'woocommerce_license_keys_completed_action_' . $action, $item, $order );
            }
        } catch ( Exception $e ) {
            Log::error( $e );
        }
    }
    /**
     * Expires generated license keys.
     * Action "woocommerce_order_status_cancelled"
     * Action "woocommerce_order_status_refunded"
     * Action "woocommerce_order_status_failed"
     * @since 1.1.0
     *
     * @param int $order_id Order ID.
     */
    public function order_cancelled( $order_id )
    {
        try {
            $license_keys = [];
            foreach ( apply_filters( 'woocommerce_license_keys_to_cancel', LicenseKey::fromOrder( $order_id ), $order_id ) as $license_key ) {
                // Force expiration
                $license_key = apply_filters( 'woocommerce_license_key', LicenseKey::findByCode( $license_key->order_item_id, $license_key->code ) );
                $license_key->expire = strtotime( '-1 day' );
                $license_key = apply_filters( 'woocommerce_license_key_before_save', $license_key );
                $license_key->save();
                add_action( 'woocommerce_license_key_saved', $license_key );
                do_action( 'woocommerce_license_key_cancelled', $license_key, $order_id );
                $license_keys[] = $license_key;
            }
            // Add remove note
            if ( !empty( $license_keys ) ) {
                $order = wc_get_order(  $order_id );
                $order->add_order_note( apply_filters(
                    'woocommerce_license_key_remove_note',
                    sprintf(
                        __( 'License key(s) %s have been set as expired.', 'woo-license-keys' ),
                        implode( ', ', array_map( function( $license_key ) { return($license_key->the_key); }, $license_keys ) )
                    ),
                    $license_keys
                ), 0 );
                $order->save();
            }
        } catch ( Exception $e ) {
            Log::error( $e );
        }
    }
    /**
     * Returns list of product tabs (PRODUCT PAGE).
     * Filter "woocommerce_product_tabs"
     * @since 1.0.0
     *
     * @see https://docs.woocommerce.com/document/editing-product-data-tabs/
     *
     * @global object $product Current product being displayed.
     *
     * @param array $tabs Product tabs.
     *
     * @return array
     */
    public function product_tabs( $tabs = [] )
    {
        global $product;
        if ( isset( $product ) && wc_is_license_key( $product ) ) {
            $tabs['license_key_details'] = [
                'title'     => apply_filters( 'woocommerce_license_key_details_heading', __( 'License Details', 'woo-license-keys' ) ),
                'priority'  => apply_filters( 'woocommerce_license_key_tab_details_priority', 15 ),
                'callback'  => [&$this, 'tab_details'],
            ];
        }
        return $tabs;
    }
    /**
     * Displays license key details tab.
     * @since 1.0.0
     *
     * @see https://docs.woocommerce.com/document/editing-product-data-tabs/
     *
     * @global object $product Current product being displayed.
     *
     * @param string $tab Tab key
     */
    public function tab_details( $tab )
    {
        global $product;
        // Display template
        wc_get_template(
            'single-product/tabs/license-key-details.php',
            [
                'tab'           => $tab,
                'heading'       => esc_html( apply_filters( 'woocommerce_license_key_details_heading', __( 'License Details', 'woo-license-keys' ) ) ),
                'license_key'   => $this->get_default_license_key( $product )
            ],
            null,
            __DIR__.'/../../templates/'
        );
    }
    /**
     * Returns default attributes of a license key, based on the attributes found in a product.
     * @since 1.0.0
     * @since 1.1.0 Fixes woo warning.
     * @since 1.1.2 Added is_desktop.
     *
     * @param int $prodcut Product.
     *
     * @return LicenseKey
     */
    public function get_default_license_key( $product )
    {
        $data = [
            'product'       => $product,
            'is_desktop'    => get_post_meta( $product->get_id(), '_desktop', true ) === 'yes',
            'expire'        => get_post_meta( $product->get_id(), '_expire', true ) === 'yes',
            'limit'         => ! empty( get_post_meta( $product->get_id(), '_limit', true ) ),
        ];
        // Add extra details
        if ( $data['expire'] ) {
            $data['expire_interval'] = get_post_meta( $product->get_id(), '_expire_interval', true );
            $data['expire_value'] = get_post_meta( $product->get_id(), '_expire_value', true );
        }
        if ( $data['limit'] ) {
            $data['limit'] = [
                get_post_meta( $product->get_id(), '_limit', true ) => get_post_meta( $product->get_id(), '_limit_value', true )
            ];
        }
        return apply_filters( 'woocommerce_default_license_key', new LicenseKey( $data ) );
    }
    /**
     * Returns api section settings.
     * @since 1.0.0
     *
     * @see https://docs.woocommerce.com/document/adding-a-section-to-a-settings-tab/
     *
     * @param array $sections Settings sections.
     *
     * @return array
     */
    public function sections_api( $sections = [] )
    {
        $sections['license_keys'] = __( 'License Keys API', 'woo-license-keys' );
        return $sections;   
    }
    /**
     * Returns License Key settings section data.
     * @since 1.0.0
     * @since 1.2.0 Added header settings.
     * @since 1.2.3 Updated text domain.
     *
     * @param array  $settings        Settings.
     * @param string $current_section Current section key.
     *
     * @return array
     */
    public function settings_api( $settings = [], $current_section )
    {
        if ( $current_section === 'license_keys' ) {
            return apply_filters( 'woocommerce_license_key_settings', [
                [
                    'name'  => __( 'Store Code', 'woo-license-keys' ),
                    'type'  => 'title',
                    'desc'  => $this->view->get( 'admin.woocommerce.settings-store-code' ),
                    'id'    => 'license_keys_store_code',
                ],
                [
                    'type'  => 'sectionend',
                    'id'    => 'license_keys_store_code',
                ],
                [
                    'name'  => __( 'HTTP Headers', 'woo-license-keys' ),
                    'type'  => 'title',
                    'id'    => 'license_keys_headers',
                ],
                [
                    'name'  => __( 'Enable override', 'woo-license-keys' ),
                    'id'    => 'license_keys_override_headers',
                    'type'  => 'checkbox',
                    'desc'  => __( 'Check this option to override Wordpress default headers with the ones below', 'woo-license-keys' ),
                    'desc_tip' => sprintf(
                                __( 'Only effective during API calls; this option is useful if you want to have control over %s.', 'woo-license-keys' ),
                                '<a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/CORS" target="_blank">CORS</a>'
                            ),
                ],
                [
                    'name'  => __( 'Allow Origin', 'woo-license-keys' ),
                    'id'    => 'license_keys_header_allow_origin',
                    'type'  => 'text',
                    'desc_tip' => sprintf( __( 'Value for "%s".', 'woo-license-keys' ), 'Access-Control-Allow-Origin' ),
                    'default' => '*',
                ],
                [
                    'name'  => __( 'Allow Methods', 'woo-license-keys' ),
                    'id'    => 'license_keys_header_allow_methods',
                    'type'  => 'text',
                    'desc_tip' => sprintf( __( 'Value for "%s".', 'woo-license-keys' ), 'Access-Control-Allow-Methods' ),
                    'default' => 'POST, GET',
                ],
                [
                    'name'  => __( 'Allow Credentials', 'woo-license-keys' ),
                    'id'    => 'license_keys_header_allow_credentials',
                    'type'  => 'select',
                    'options' => apply_filters( 'woocommerce_license_key_allow_credentials',
                            [
                                ''      => __( '-None-', 'woo-license-keys' ),
                                'true'  => __( 'True', 'woo-license-keys' ),
                                'false' => __( 'False', 'woo-license-keys' ),
                            ] ),
                    'desc_tip' => sprintf( __( 'Value for "%s".', 'woo-license-keys' ), 'Access-Control-Allow-Credentials' ),
                    'default' => 'true',
                ],
                [
                    'type'  => 'sectionend',
                    'id'    => 'license_keys_headers',
                ],
                [
                    'name'  => __( 'HTTP Response', 'woo-license-keys' ),
                    'type'  => 'title',
                    'id'    => 'license_keys_response',
                ],
                [
                    'name'  => __( 'Errors output', 'woo-license-keys' ),
                    'id'    => 'license_keys_response_errors_format',
                    'type'  => 'select',
                    'options' => apply_filters( 'woocommerce_license_key_response_error_formats',
                            [
                                'property'  => __( 'Grouped by property name.', 'woo-license-keys' ),
                                'code'      => __( 'Associated to an error code.', 'woo-license-keys' ),
                            ] ),
                    'desc_tip' => __( 'Indicates how should errors be returned in API responses.', 'woo-license-keys' ),
                    'default' => 'property',
                ],
                [
                    'type'  => 'sectionend',
                    'id'    => 'license_keys_response',
                ],
                [
                    'name'  => __( 'DOCUMENTATION', 'woo-license-keys' ),
                    'type'  => 'title',
                    'desc'  => $this->view->get( 'admin.woocommerce.settings-license-key' ),
                    'id'    => 'license_keys_documentation',
                ],
                [
                    'type'  => 'sectionend',
                    'id'    => 'license_keys_documentation',
                ],
            ] );
        }
        return $settings;
    }
    /**
     * Displays license keys on order complete email.
     * Action "woocommerce_email_after_order_table"
     * @since 1.0.0
     * @since 1.1.2 Fixes doing it wrong.
     *
     * @see http://hookr.io/actions/woocommerce_email_after_order_table/
     *
     * @param WC_Order $order         Order data object.
     * @param bool     $sent_to_admin Flag that indicates if this is being sent to admin.
     * @param bool     $plain_text    Flag that indicates if this is a plain text email.
     * @param objecy   $email         Email object.
     */
    public function email( $order, $sent_to_admin, $plain_text, $email )
    {
        if ( $email->id === 'customer_completed_order' ) {
            $license_keys = apply_filters( 'woocommerce_license_keys_in_email', LicenseKey::fromOrder( $order->get_id() ), $order->get_id() );
            if ( count( $license_keys ) > 0 ) {
                // Display template
                wc_get_template(
                    $plain_text === true ? 'emails/plain/email-license-keys.php' : 'emails/email-license-keys.php',
                    [
                        'license_keys'  => $license_keys,
                        'text_align'    => is_rtl() ? 'right' : 'left',
                    ],
                    null,
                    __DIR__.'/../../templates/'
                );
            }
        }
    }
    /**
     * Displays license keys on order received / completed.
     * Action "woocommerce_thankyou"
     * @since 1.0.2
     *
     * @param int $order_id Order ID.
     */
    public function thankyou( $order_id )
    {
        $license_keys = apply_filters( 'woocommerce_license_keys_in_thankyou', LicenseKey::fromOrder( $order_id ), $order_id );
        if ( count( $license_keys ) > 0 ) {
            // Display template
            wc_get_template(
                'order/order-license-keys.php',
                [
                    'license_keys'  => $license_keys,
                ],
                null,
                __DIR__.'/../../templates/'
            );
        }
    }
    /**
     * Returns list of license key based product types.
     * @since 1.2.9
     * 
     * @hook woocommerce_license_key_types
     * 
     * @return array
     */
    public function types()
    {
        return [WC_Product_License_Key::TYPE];
    }
    /**
     * Returns the options checkboxes available per product.
     * @since 1.2.9
     * 
     * @hook product_type_options
     * 
     * @param array $options
     * 
     * @return array
     */
    public function product_type_options( $options )
    {
        $types = apply_filters( 'woocommerce_license_key_types', [] );
        $options['virtual']['wrapper_class'] .= ' '
            . wc_lk_show_if( apply_filters( 'woocommerce_license_key_types_show_if_virtual', $types ) );
        $options['downloadable']['wrapper_class'] .= ' '
            . wc_lk_show_if( apply_filters( 'woocommerce_license_key_types_show_if_downloadable', $types ) );
        return $options;
    }
}