<?php

namespace LicenseKeys\Traits;

use WPMVC\MVC\Collection;

/**
 * License Key find options trait.
 *
 * @author Cami Mostajo <info@10quality.com>
 * @copyright 10 Quality <http://www.10quality.com/>
 * @license GPLv3
 * @package LicenseKeys
 * @version 1.2.9
 */
trait FindTrait
{
    /**
     * Finds license key by Order item ID and small code.
     * @since 1.0.0
     * @since 1.0.8 Removed status filter.
     * @since 1.1.0 Check license key validity.
     *
     * @global $wpdb Wordpress Database accessor.
     *
     * @param int    $item_id Order Item ID.
     * @param string $code    Small license key code.
     *
     * @return mixed|null|object
     */
    public static function findByCode( $item_id, $code )
    {
        global $wpdb;
        $results = $wpdb->get_results( preg_replace( '/{w}/', '%', $wpdb->prepare(
            'SELECT items.order_id,
                items.order_item_id,
                metaproduct.meta_value as product_id,
                meta.meta_id
            FROM ' . $wpdb->posts . ' AS posts
            INNER JOIN ' . $wpdb->prefix . 'woocommerce_order_items AS items
                ON items.order_id = posts.ID
                AND items.order_item_id = %d
            INNER JOIN ' . $wpdb->prefix . 'woocommerce_order_itemmeta AS metaproduct
                ON metaproduct.order_item_id = items.order_item_id
                AND metaproduct.meta_key = %s
            INNER JOIN ' . $wpdb->prefix . 'woocommerce_order_itemmeta AS meta
                ON meta.order_item_id = items.order_item_id
                AND meta.meta_key = %s
                AND meta.meta_value LIKE %s
            WHERE posts.post_type = %s',
            $item_id,
            '_product_id',
            '_license_key',
            '{w}"'.$wpdb->esc_like( $code ).'"{w}',
            'shop_order'
        ) ) );
        foreach ( $results as $row ) {
            // Get meta
            $keys = wc_get_order_item_meta( $row->order_item_id, '_license_key', false );
            $product = wc_get_product( $row->product_id );
            if ( ! is_array( $keys ) )
                continue;
            if ( ! array_key_exists( 0, $keys ) )
                $keys = [$keys];
            for ( $i = 0; $i < count( $keys ); ++$i ) {
                // Add index
                if ( $keys[$i]['code'] === $code ) {
                    $keys[$i]['key_index'] = $i;
                    $keys[$i]['order_id'] = $row->order_id;
                    $keys[$i]['order_item_id'] = $row->order_item_id;
                    $keys[$i]['meta_id'] = $row->meta_id;
                    $keys[$i]['product'] = $product;
                    // Return license key
                    $license_key = new self( $keys[$i] );
                    return $license_key->is_valid ? $license_key : null;
                }
            }
        }
        return;
    }
    /**
     * Finds by license key.
     * @since 1.0.0
     * @param string $key License key.
     *
     * @return mixed|null|object
     */
    public static function find( $key )
    {
        $key = explode( '-', $key );
        if ( count( $key ) === 2 ) {
            return self::findByCode( $key[1], $key[0] );
        }
        return;
    }
    /**
     * Finds license key by user and index.
     * @since 1.0.0
     * @since 1.1.0 Check license key validity.
     * @since 1.2.9 Added hooks.
     *
     * @global $wpdb Wordpress Database accessor.
     *
     * @param int    $user_id User ID.
     * @param int    $item_id Order Item ID.
     * @param string $index   License key index in order item.
     *
     * @return mixed|null|object
     */
    public static function findByUserIndex( $user_id, $item_id, $index = 0 )
    {
        global $wpdb;
        $status = apply_filters( 'woocommerce_license_keys_query_order_status', ['wc-completed'] );
        $results = $wpdb->get_results( $wpdb->prepare(
            'SELECT items.order_id,
                items.order_item_id,
                metaproduct.meta_value as product_id
            FROM ' . $wpdb->posts . ' AS posts
            INNER JOIN ' . $wpdb->postmeta . ' as postmeta
                ON postmeta.post_id = posts.ID
                AND postmeta.meta_key = %s
                AND postmeta.meta_value = %d
            INNER JOIN ' . $wpdb->prefix . 'woocommerce_order_items AS items
                ON items.order_id = posts.ID
                AND items.order_item_id = %d
            INNER JOIN ' . $wpdb->prefix . 'woocommerce_order_itemmeta AS metaproduct
                ON metaproduct.order_item_id = items.order_item_id
                AND metaproduct.meta_key = %s
            INNER JOIN ' . $wpdb->prefix . 'woocommerce_order_itemmeta AS meta
                ON meta.order_item_id = items.order_item_id
                AND meta.meta_key = %s
            WHERE posts.post_type = %s
                AND posts.post_status IN (\'' . implode( '\',\'' , $status ) . '\')
            GROUP BY order_item_id
            ORDER BY meta.meta_id',
            '_customer_user',
            $user_id,
            intval( $item_id ),
            '_product_id',
            '_license_key',
            'shop_order'
        ) );
        foreach ( $results as $row ) {
            // Get meta
            $keys = wc_get_order_item_meta( $row->order_item_id, '_license_key', false );
            $product = wc_get_product( $row->product_id );
            if ( ! is_array( $keys ) )
                continue;
            if ( ! array_key_exists( 0, $keys ) )
                $keys = [$keys];
            for ( $i = 0; $i < count( $keys ); ++$i ) {
                // Add index
                if ( intval( $index ) === $i ) {
                    $keys[$i]['key_index'] = $i;
                    $keys[$i]['order_id'] = $row->order_id;
                    $keys[$i]['order_item_id'] = $row->order_item_id;
                    $keys[$i]['product'] = $product;
                    // Return license key
                    $license_key = new self( $keys[$i] );
                    return $license_key->is_valid ? $license_key : null;
                }
            }
        }
        return;
    }
    /**
     * Finds license key by user and index.
     * @since 1.0.0
     * @since 1.1.0 Check license key validity.
     *
     * @global $wpdb Wordpress Database accessor.
     *
     * @param int    $user_id User ID.
     * @param int    $item_id Order Item ID.
     * @param string $index   License key index in order item.
     *
     * @return mixed|null|object
     */
    public static function findByUserKey( $user_id, $key )
    {
        $key = explode( '-', $key );
        if ( count( $key ) === 2 ) {
            global $wpdb;
            $status = apply_filters( 'woocommerce_license_keys_query_order_status', ['wc-completed'] );
            $results = $wpdb->get_results( preg_replace( '/{w}/', '%', $wpdb->prepare(
                'SELECT items.order_id,
                    items.order_item_id,
                    metaproduct.meta_value as product_id,
                    meta.meta_id
                FROM ' . $wpdb->posts . ' AS posts
                INNER JOIN ' . $wpdb->postmeta . ' as postmeta
                    ON postmeta.post_id = posts.ID
                    AND postmeta.meta_key = %s
                    AND postmeta.meta_value = %d
                INNER JOIN ' . $wpdb->prefix . 'woocommerce_order_items AS items
                    ON items.order_id = posts.ID
                    AND items.order_item_id = %d
                INNER JOIN ' . $wpdb->prefix . 'woocommerce_order_itemmeta AS metaproduct
                    ON metaproduct.order_item_id = items.order_item_id
                    AND metaproduct.meta_key = %s
                INNER JOIN ' . $wpdb->prefix . 'woocommerce_order_itemmeta AS meta
                    ON meta.order_item_id = items.order_item_id
                    AND meta.meta_key = %s
                    AND meta.meta_value LIKE %s
                WHERE posts.post_type = %s
                    AND posts.post_status IN (\'' . implode( '\',\'' , $status ) . '\')',
                '_customer_user',
                $user_id,
                intval( $key[1] ),
                '_product_id',
                '_license_key',
                '{w}"'.$wpdb->esc_like( $key[0] ).'"{w}',
                'shop_order'
            ) ) );
            foreach ( $results as $row ) {
                // Get meta
                $keys = wc_get_order_item_meta( $row->order_item_id, '_license_key', false );
                $product = wc_get_product( $row->product_id );
                if ( ! is_array( $keys ) )
                    continue;
                if ( ! array_key_exists( 0, $keys ) )
                    $keys = [$keys];
                for ( $i = 0; $i < count( $keys ); ++$i ) {
                    // Add index
                    if ( $keys[$i]['code'] === $key[0]) {
                        $keys[$i]['order_id'] = $row->order_id;
                        $keys[$i]['order_item_id'] = $row->order_item_id;
                        $keys[$i]['meta_id'] = $row->meta_id;
                        $keys[$i]['product'] = $product;
                        // Return license key
                        $license_key = new self( $keys[$i] );
                        return $license_key->is_valid ? $license_key : null;
                    }
                }
            }
        }
        return;
    }
    /**
     * Returns a collection of license keys related to a user.
     * @since 1.0.0
     * @since 1.1.0 Check license key validity.
     *
     * @global $wpdb Wordpress Database accessor.
     *
     * @param int $user_id User ID.
     *
     * @return Collection
     */
    public static function fromUser( $user_id )
    {
        global $wpdb;
        $license_keys = new Collection;
        $status = apply_filters( 'woocommerce_license_keys_query_order_status', ['wc-completed'] );
        $results = $wpdb->get_results( $wpdb->prepare(
            'SELECT items.order_id,
                items.order_item_id,
                metaproduct.meta_value as product_id,
                count(2) as `count`
            FROM ' . $wpdb->posts . ' AS posts
            INNER JOIN ' . $wpdb->postmeta . ' as postmeta
                ON postmeta.post_id = posts.ID
                AND postmeta.meta_key = %s
                AND postmeta.meta_value = %d
            INNER JOIN ' . $wpdb->prefix . 'woocommerce_order_items AS items
                ON items.order_id = posts.ID
            INNER JOIN ' . $wpdb->prefix . 'woocommerce_order_itemmeta AS metaproduct
                ON metaproduct.order_item_id = items.order_item_id
                AND metaproduct.meta_key = %s
            INNER JOIN ' . $wpdb->prefix . 'woocommerce_order_itemmeta AS meta
                ON meta.order_item_id = items.order_item_id
                AND meta.meta_key = %s
            WHERE posts.post_type = %s
                AND posts.post_status IN (\'' . implode( '\',\'' , $status ) . '\')
            GROUP BY order_id, order_item_id
            ORDER BY posts.post_date DESC, meta.meta_id',
            '_customer_user',
            $user_id,
            '_product_id',
            '_license_key',
            'shop_order'
        ) );
        foreach ( $results as $row ) {
            // Get meta
            $keys = wc_get_order_item_meta( $row->order_item_id, '_license_key', false );
            $product = wc_get_product( $row->product_id );
            if ( ! is_array( $keys ) )
                continue;
            if ( ! array_key_exists( 0, $keys ) )
                $keys = [$keys];
            for ( $i = 0; $i < count( $keys ); ++$i ) {
                // Add index
                $keys[$i]['key_index'] = $i;
                $keys[$i]['order_id'] = $row->order_id;
                $keys[$i]['order_item_id'] = $row->order_item_id;
                $keys[$i]['product'] = $product;
                $license_key = new self( $keys[$i] );
                if ( $license_key->is_valid )
                    $license_keys[] = $license_key;
            }
        }
        return $license_keys;
    }
    /**
     * Returns a collection of license keys related to a user.
     * @since 1.0.0
     * @since 1.1.0 Check license key validity.
     *
     * @param int $item_id Order Item ID.
     *
     * @return Collection
     */
    public static function findByItem( $item_id )
    {
        $license_keys = new Collection;
        // Get meta
        $keys = wc_get_order_item_meta( $item_id, '_license_key', false );
        if ( ! is_array( $keys ) )
            return;
        if ( ! array_key_exists( 0, $keys ) )
            $keys = [$keys];
        for ( $i = 0; $i < count( $keys ); ++$i ) {
            // Add index
            $keys[$i]['key_index'] = $i;
            $keys[$i]['order_item_id'] = $item_id;
            $license_key = new self( $keys[$i] );
            if ( $license_key->is_valid )
                $license_keys[] = $license_key;
        }
        return $license_keys;
    }
    /**
     * Returns a collection of license keys related to an order.
     * @since 1.0.0
     * @since 1.0.8 Removed status filter.
     * @since 1.1.0 Check license key validity.
     *
     * @global $wpdb Wordpress Database accessor.
     *
     * @param int $order_id Order ID.
     *
     * @return Collection
     */
    public static function fromOrder( $order_id )
    {
        global $wpdb;
        $license_keys = new Collection;
        $results = $wpdb->get_results( $wpdb->prepare(
            'SELECT items.order_id,
                items.order_item_id,
                metaproduct.meta_value as product_id,
                count(2) as `count`
            FROM ' . $wpdb->posts . ' AS posts
            INNER JOIN ' . $wpdb->prefix . 'woocommerce_order_items AS items
                ON items.order_id = posts.ID
            INNER JOIN ' . $wpdb->prefix . 'woocommerce_order_itemmeta AS metaproduct
                ON metaproduct.order_item_id = items.order_item_id
                AND metaproduct.meta_key = %s
            INNER JOIN ' . $wpdb->prefix . 'woocommerce_order_itemmeta AS meta
                ON meta.order_item_id = items.order_item_id
                AND meta.meta_key = %s
            WHERE posts.post_type = %s
                AND posts.ID = %d
            GROUP BY order_id, order_item_id
            ORDER BY posts.post_date DESC, meta.meta_id',
            '_product_id',
            '_license_key',
            'shop_order',
            $order_id
        ) );
        foreach ( $results as $row ) {
            // Get meta
            $keys = wc_get_order_item_meta( $row->order_item_id, '_license_key', false );
            $product = wc_get_product( $row->product_id );
            if ( ! is_array( $keys ) )
                continue;
            if ( ! array_key_exists( 0, $keys ) )
                $keys = [$keys];
            for ( $i = 0; $i < count( $keys ); ++$i ) {
                // Add index
                $keys[$i]['key_index'] = $i;
                $keys[$i]['order_id'] = $row->order_id;
                $keys[$i]['order_item_id'] = $row->order_item_id;
                $keys[$i]['product'] = $product;
                $license_key = new self( $keys[$i] );
                if ( $license_key->is_valid )
                    $license_keys[] = $license_key;
            }
        }
        return $license_keys;
    }
    /**
     * Returns a collection of license keys related to an order item.
     * @since 1.1.0
     *
     * @global $wpdb Wordpress Database accessor.
     *
     * @param int $order_id      Order ID.
     * @param int $order_item_id Order item ID.
     *
     * @return Collection
     */
    public static function fromOrderItem( $order_id, $order_item_id )
    {
        global $wpdb;
        $license_keys = new Collection;
        // Get meta
        $keys = wc_get_order_item_meta( $order_item_id, '_license_key', false );
        $product = wc_get_product( wc_get_order_item_meta( $order_item_id, '_product_id' ) );
        if ( ! is_array( $keys ) )
            return $license_keys;
        if ( ! array_key_exists( 0, $keys ) )
            $keys = [$keys];
        for ( $i = 0; $i < count( $keys ); ++$i ) {
            // Add index
            $keys[$i]['key_index'] = $i;
            $keys[$i]['order_id'] = $order_id;
            $keys[$i]['order_item_id'] = $order_item_id;
            $keys[$i]['product'] = $product;
            $license_key = new self( $keys[$i] );
            if ( $license_key->is_valid )
                $license_keys[] = $license_key;
        }
        return $license_keys;
    }
}